window.Gleam.initSearch([{"doc":"gleam_bson","title":"gleam_bson","content":"# gleam_bson\n\nbson encoder and decoder for gleam\n\n## Quick start\n\n```sh\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\n```sh\ngleam add gleam_bson\n```\n\n## Roadmap\n\n- [x] support encoding and decoding basic bson types (null, string, int32, int64, double, boolean, objectId, array, document).\n- [x] support encoding and decoding min, max, timestamp, datetime, javascript and regex bson types.\n- [x] support encoding and decoding the generic, md5, uuid and user-defined binary subtypes.\n- [x] support generating new object-ids\n- [ ] support encoding and decoding decimal128 bson type.\n- [ ] support encoding and decoding encrypted binary subtype.\n\n## Usage\n\n### Encoding\n\n```gleam\nimport gleam/list\nimport gleam/result\nimport bson/md5\nimport bson/value\nimport bson.{encode}\nimport bson/object_id\n\nfn cat_to_bson(cat: Cat) -&gt; Result(BitString, Nil) {\n  use id &lt;- result.then(object_id.from_string(cat.id))\n  use checksum &lt;- result.then(md5.from_string(cat.checksum))\n\n  Ok(encode([\n    #(&quot;id&quot;, value.ObjectId(id)),\n    #(&quot;name&quot;, value.Str(cat.name)),\n    #(&quot;lives&quot;, value.Integer(cat.lives)),\n    #(&quot;nicknames&quot;, value.Array(list.map(cat.nicknames, value.Str))),\n    #(&quot;checksum&quot;, value.Binary(value.MD5(checksum))),\n    #(&quot;name_pattern&quot;, value.Regex(#(&quot;[a-z][a-z0-9]+&quot;, &quot;&quot;))),\n  ]))\n}\n```\n\n### Decoding\n\n```gleam\nimport gleam/list\nimport gleam/result\nimport bson/md5\nimport bson/value\nimport bson.{decode}\nimport bson/object_id\n\nfn cat_from_bson(data: BitString) -&gt; Result(Cat, Nil) {\n  use doc &lt;- result.then(decode(data))\n\n  let [\n    #(&quot;id&quot;, value.ObjectId(id)),\n    #(&quot;name&quot;, value.Str(name)),\n    #(&quot;lives&quot;, value.Integer(lives)),\n    #(&quot;nicknames&quot;, value.Array(nicknames)),\n    #(&quot;checksum&quot;, value.Binary(value.MD5(checksum))),\n    #(&quot;name_pattern&quot;, value.Regex(#(pattern, options))),\n  ] = doc\n\n  Ok(Cat(\n    id: object_id.to_string(id),\n    name: name,\n    lives: lives,\n    nicknames: list.map(\n      nicknames,\n      fn(n) {\n        let assert value.Str(n) = n\n        n\n      },\n    ),\n    checksum: md5.to_string(checksum),\n  ))\n}\n```\n","url":"index.html"},{"doc":"bson/md5","title":"MD5","content":"pub opaque type MD5\n\n\nSynonyms:\nbson/md5.MD5\nbson/md5 MD5","url":"bson/md5.html#MD5"},{"doc":"bson/md5","title":"from_bit_string","content":"pub fn from_bit_string(md5: BitString) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_bit_string\nbson/md5 from_bit_string","url":"bson/md5.html#from_bit_string"},{"doc":"bson/md5","title":"from_int_list","content":"pub fn from_int_list(md5: List(Int)) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_int_list\nbson/md5 from_int_list","url":"bson/md5.html#from_int_list"},{"doc":"bson/md5","title":"from_string","content":"pub fn from_string(md5: String) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_string\nbson/md5 from_string","url":"bson/md5.html#from_string"},{"doc":"bson/md5","title":"to_bit_string","content":"pub fn to_bit_string(md5: MD5) -&gt; BitString\n\nSynonyms:\nbson/md5.to_bit_string\nbson/md5 to_bit_string","url":"bson/md5.html#to_bit_string"},{"doc":"bson/md5","title":"to_int_list","content":"pub fn to_int_list(md5: MD5) -&gt; List(Int)\n\nSynonyms:\nbson/md5.to_int_list\nbson/md5 to_int_list","url":"bson/md5.html#to_int_list"},{"doc":"bson/md5","title":"to_string","content":"pub fn to_string(md5: MD5) -&gt; String\n\nSynonyms:\nbson/md5.to_string\nbson/md5 to_string","url":"bson/md5.html#to_string"},{"doc":"bson/md5","title":"bson/md5","content":"","url":"bson/md5.html"},{"doc":"bson/uuid","title":"UUID","content":"pub opaque type UUID\n\n\nSynonyms:\nbson/uuid.UUID\nbson/uuid UUID","url":"bson/uuid.html#UUID"},{"doc":"bson/uuid","title":"from_bit_string","content":"pub fn from_bit_string(uuid: BitString) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_bit_string\nbson/uuid from_bit_string","url":"bson/uuid.html#from_bit_string"},{"doc":"bson/uuid","title":"from_int_list","content":"pub fn from_int_list(uuid: List(Int)) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_int_list\nbson/uuid from_int_list","url":"bson/uuid.html#from_int_list"},{"doc":"bson/uuid","title":"from_string","content":"pub fn from_string(uuid: String) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_string\nbson/uuid from_string","url":"bson/uuid.html#from_string"},{"doc":"bson/uuid","title":"to_bit_string","content":"pub fn to_bit_string(uuid: UUID) -&gt; BitString\n\nSynonyms:\nbson/uuid.to_bit_string\nbson/uuid to_bit_string","url":"bson/uuid.html#to_bit_string"},{"doc":"bson/uuid","title":"to_int_list","content":"pub fn to_int_list(uuid: UUID) -&gt; List(Int)\n\nSynonyms:\nbson/uuid.to_int_list\nbson/uuid to_int_list","url":"bson/uuid.html#to_int_list"},{"doc":"bson/uuid","title":"to_string","content":"pub fn to_string(uuid: UUID) -&gt; String\n\nSynonyms:\nbson/uuid.to_string\nbson/uuid to_string","url":"bson/uuid.html#to_string"},{"doc":"bson/uuid","title":"bson/uuid","content":"","url":"bson/uuid.html"},{"doc":"bson/custom","title":"Custom","content":"pub opaque type Custom\n\n\nSynonyms:\nbson/custom.Custom\nbson/custom Custom","url":"bson/custom.html#Custom"},{"doc":"bson/custom","title":"max_binary_custom_code","content":"pub const max_binary_custom_code: Int = 0xff\n\nSynonyms:\nbson/custom.max_binary_custom_code\nbson/custom max_binary_custom_code","url":"bson/custom.html#max_binary_custom_code"},{"doc":"bson/custom","title":"min_binary_custom_code","content":"pub const min_binary_custom_code: Int = 0x80\n\nSynonyms:\nbson/custom.min_binary_custom_code\nbson/custom min_binary_custom_code","url":"bson/custom.html#min_binary_custom_code"},{"doc":"bson/custom","title":"from_bit_string_with_code","content":"pub fn from_bit_string_with_code(code: Int, value: BitString) -&gt; Result(\n  Custom,\n  Nil,\n)\n\nSynonyms:\nbson/custom.from_bit_string_with_code\nbson/custom from_bit_string_with_code","url":"bson/custom.html#from_bit_string_with_code"},{"doc":"bson/custom","title":"to_bit_string_with_code","content":"pub fn to_bit_string_with_code(custom: Custom) -&gt; #(\n  Int,\n  BitString,\n)\n\nSynonyms:\nbson/custom.to_bit_string_with_code\nbson/custom to_bit_string_with_code","url":"bson/custom.html#to_bit_string_with_code"},{"doc":"bson/custom","title":"bson/custom","content":"","url":"bson/custom.html"},{"doc":"bson/generic","title":"Generic","content":"pub opaque type Generic\n\n\nSynonyms:\nbson/generic.Generic\nbson/generic Generic","url":"bson/generic.html#Generic"},{"doc":"bson/generic","title":"from_bit_string","content":"pub fn from_bit_string(data: BitString) -&gt; Result(Generic, Nil)\n\nSynonyms:\nbson/generic.from_bit_string\nbson/generic from_bit_string","url":"bson/generic.html#from_bit_string"},{"doc":"bson/generic","title":"from_int_list","content":"pub fn from_int_list(data: List(Int)) -&gt; Generic\n\nSynonyms:\nbson/generic.from_int_list\nbson/generic from_int_list","url":"bson/generic.html#from_int_list"},{"doc":"bson/generic","title":"from_string","content":"pub fn from_string(data: String) -&gt; Generic\n\nSynonyms:\nbson/generic.from_string\nbson/generic from_string","url":"bson/generic.html#from_string"},{"doc":"bson/generic","title":"to_bit_string","content":"pub fn to_bit_string(generic: Generic) -&gt; BitString\n\nSynonyms:\nbson/generic.to_bit_string\nbson/generic to_bit_string","url":"bson/generic.html#to_bit_string"},{"doc":"bson/generic","title":"to_int_list","content":"pub fn to_int_list(generic: Generic) -&gt; List(Int)\n\nSynonyms:\nbson/generic.to_int_list\nbson/generic to_int_list","url":"bson/generic.html#to_int_list"},{"doc":"bson/generic","title":"to_string","content":"pub fn to_string(generic: Generic) -&gt; Result(String, Nil)\n\nSynonyms:\nbson/generic.to_string\nbson/generic to_string","url":"bson/generic.html#to_string"},{"doc":"bson/generic","title":"bson/generic","content":"","url":"bson/generic.html"},{"doc":"bson/object_id","title":"ObjectId","content":"pub opaque type ObjectId\n\n\nSynonyms:\nbson/object_id.ObjectId\nbson/object_id ObjectId","url":"bson/object_id.html#ObjectId"},{"doc":"bson/object_id","title":"compare","content":"pub fn compare(a: ObjectId, b: ObjectId) -&gt; Order\n\nSynonyms:\nbson/object_id.compare\nbson/object_id compare","url":"bson/object_id.html#compare"},{"doc":"bson/object_id","title":"from_bit_string","content":"pub fn from_bit_string(id: BitString) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_bit_string\nbson/object_id from_bit_string","url":"bson/object_id.html#from_bit_string"},{"doc":"bson/object_id","title":"from_datetime","content":"pub fn from_datetime(datetime: DateTime) -&gt; ObjectId\n see [birl](https://hex.pm/packages/birl)!\nSynonyms:\nbson/object_id.from_datetime\nbson/object_id from_datetime","url":"bson/object_id.html#from_datetime"},{"doc":"bson/object_id","title":"from_int_list","content":"pub fn from_int_list(id: List(Int)) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_int_list\nbson/object_id from_int_list","url":"bson/object_id.html#from_int_list"},{"doc":"bson/object_id","title":"from_string","content":"pub fn from_string(id: String) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_string\nbson/object_id from_string","url":"bson/object_id.html#from_string"},{"doc":"bson/object_id","title":"new","content":"pub fn new() -&gt; ObjectId\n\nSynonyms:\nbson/object_id.new\nbson/object_id new","url":"bson/object_id.html#new"},{"doc":"bson/object_id","title":"range","content":"pub fn range(from a: ObjectId, to b: Option(ObjectId), step s: Duration) -&gt; Iterator(\n  ObjectId,\n)\n can be used to create a time range starting from time `a` with step `s`,\n which is of type `duration.Duration` from package [birl](https://hex.pm/packages/birl)\n\n if `b` is `option.None` the range will be infinite\nSynonyms:\nbson/object_id.range\nbson/object_id range","url":"bson/object_id.html#range"},{"doc":"bson/object_id","title":"to_bit_string","content":"pub fn to_bit_string(id: ObjectId) -&gt; BitString\n\nSynonyms:\nbson/object_id.to_bit_string\nbson/object_id to_bit_string","url":"bson/object_id.html#to_bit_string"},{"doc":"bson/object_id","title":"to_datetime","content":"pub fn to_datetime(id: ObjectId) -&gt; DateTime\n see [birl](https://hex.pm/packages/birl)!\nSynonyms:\nbson/object_id.to_datetime\nbson/object_id to_datetime","url":"bson/object_id.html#to_datetime"},{"doc":"bson/object_id","title":"to_int_list","content":"pub fn to_int_list(id: ObjectId) -&gt; List(Int)\n\nSynonyms:\nbson/object_id.to_int_list\nbson/object_id to_int_list","url":"bson/object_id.html#to_int_list"},{"doc":"bson/object_id","title":"to_string","content":"pub fn to_string(id: ObjectId) -&gt; String\n\nSynonyms:\nbson/object_id.to_string\nbson/object_id to_string","url":"bson/object_id.html#to_string"},{"doc":"bson/object_id","title":"bson/object_id","content":"","url":"bson/object_id.html"},{"doc":"bson/value","title":"Binary","content":"pub type Binary {\n  MD5(MD5)\n  UUID(UUID)\n  Custom(Custom)\n  Generic(Generic)\n}\n\nMD5(MD5)\n\n\nUUID(UUID)\n\n\nCustom(Custom)\n\n\nGeneric(Generic)\n\n\nSynonyms:\nbson/value.Binary\nbson/value Binary","url":"bson/value.html#Binary"},{"doc":"bson/value","title":"Value","content":"pub type Value {\n  Min\n  Max\n  Null\n  JS(String)\n  Str(String)\n  Integer(Int)\n  Double(Float)\n  Boolean(Bool)\n  Binary(Binary)\n  Timestamp(Int)\n  Array(List(Value))\n  ObjectId(ObjectId)\n  DateTime(DateTime)\n  Regex(#(String, String))\n  Document(List(#(String, Value)))\n}\n if you&#39;re not familiar with type `DateTime`, see [birl](https://hex.pm/packages/birl)!\nMin\n\n\nMax\n\n\nNull\n\n\nJS(String)\n\n\nStr(String)\n\n\nInteger(Int)\n\n\nDouble(Float)\n\n\nBoolean(Bool)\n\n\nBinary(Binary)\n\n\nTimestamp(Int)\n\n\nArray(List(Value))\n\n\nObjectId(ObjectId)\n\n\nDateTime(DateTime)\n\n\nRegex(#(String, String))\n\n\nDocument(List(#(String, Value)))\n\n\nSynonyms:\nbson/value.Value\nbson/value Value","url":"bson/value.html#Value"},{"doc":"bson/value","title":"bson/value","content":"","url":"bson/value.html"},{"doc":"bson/encoder","title":"encode","content":"pub fn encode(doc: List(#(String, Value))) -&gt; BitString\n\nSynonyms:\nbson/encoder.encode\nbson/encoder encode","url":"bson/encoder.html#encode"},{"doc":"bson/encoder","title":"bson/encoder","content":"","url":"bson/encoder.html"},{"doc":"bson/decoder","title":"decode","content":"pub fn decode(data: BitString) -&gt; Result(\n  List(#(String, Value)),\n  Nil,\n)\n\nSynonyms:\nbson/decoder.decode\nbson/decoder decode","url":"bson/decoder.html#decode"},{"doc":"bson/decoder","title":"bson/decoder","content":"","url":"bson/decoder.html"},{"doc":"bson","title":"decode","content":"pub fn decode(data: BitString) -&gt; Result(\n  List(#(String, Value)),\n  Nil,\n)\n\nSynonyms:\nbson.decode\nbson decode","url":"bson.html#decode"},{"doc":"bson","title":"encode","content":"pub fn encode(doc: List(#(String, Value))) -&gt; BitString\n\nSynonyms:\nbson.encode\nbson encode","url":"bson.html#encode"},{"doc":"bson","title":"bson","content":"","url":"bson.html"}]);